// -*- java -*-
/***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1/GPL 2.0/LGPL 2.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Initial Developer of the Original Code is Shawn Betts.
Portions created by the Initial Developer are Copyright (C) 2004
by the Initial Developer. All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of
either the GNU General Public License Version 2 or later (the "GPL"), or
the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
in which case the provisions of the GPL or the LGPL are applicable instead
of those above. If you wish to allow use of your version of this file only
under the terms of either the GPL or the LGPL, and not to allow others to
use your version of this file under the terms of the MPL, indicate your
decision by deleting the provisions above and replace them with the notice
and other provisions required by the GPL or the LGPL. If you do not delete
the provisions above, a recipient may use your version of this file under
the terms of any one of the MPL, the GPL or the LGPL.
***** END LICENSE BLOCK *****/

// var gNumbersOn = false;

const gDocumentObserver = {
    observe: function(subject, topic, data)
    {
	if (topic == "EndDocumentLoad") {
	    setNumberedLinksState(true);
	}
    }
};


var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
observerService.addObserver(gDocumentObserver, "StartDocumentLoad", false);
observerService.addObserver(gDocumentObserver, "EndDocumentLoad", false);

function openNumberedLinkBar() {
    var bar = document.getElementById("numberedlink-toolbar");
    if (bar.hidden) {
	bar.hidden = false;
    }
    try {
	var findfield = document.getElementById("numberedlink-field");
	findfield.focus();
    } catch(e) {
	window.alert(e);
    }
}

function closeNumberedLinkBar() {
    try {
	var findField = document.getElementById("numberedlink-field");
	var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
	    .getService(Components.interfaces.nsIWindowWatcher);
	if (window == ww.activeWindow && document.commandDispatcher.focusedElement &&
	    document.commandDispatcher.focusedElement.parentNode.parentNode == findField) {
	    _content.focus();
	}
    } catch(e) {
	window.alert(e);
    }
    var bar = document.getElementById("numberedlink-toolbar");
    if (!bar.hidden) {
	bar.hidden = true;
    }
}

function selectNumberedLink(num) {
    openNumberedLinkBar();
    var field = document.getElementById('numberedlink-field');
    field.value = num;
}

function onNumberedLinkKeyPress(evt) {
    try {
    var doc = window.content.document;
    if (!doc) return;

    if (evt.keyCode == KeyEvent.DOM_VK_RETURN) {
	var findfield = document.getElementById("numberedlink-field");
	var link = findfield.value;
	var nodes = getPageLinkNodes();
	for (var i=0; i<nodes.length; i++) {
	    if (link == i+1) {
		if (nodes[i].tagName == "A") {
		    if (evt.altKey)
			nodes[i].focus();
		    else if (evt.ctrlKey)
			delayedOpenTab(nodes[i].href);
		    else
			getWebNavigation().loadURI(nodes[i].href, 
						   nsIWebNavigation.LOAD_FLAGS_NONE, 
						   null, null, null);
		    } else {
			if (nodes[i].tagName == "INPUT" 
			    && (nodes[i].type == "submit" || nodes[i].type == "button")) {
			if (evt.altKey)
			    nodes[i].focus();
			else
			    nodes[i].click();
		    } else 
			nodes[i].focus();
		    }
		break;
	    }
	}
	closeNumberedLinkBar();
    }
    else if (evt.keyCode == KeyEvent.DOM_VK_TAB 
	     || evt.keyCode == KeyEvent.DOM_VK_ESCAPE) {
	closeNumberedLinkBar();
	evt.preventDefault();
    }
    } catch(e) {alert(e);}
}

function onNumberedLinkBlur() {

}

function toggleNum(node, n, TurnOn)
{
    try{
    var doc = window.content.document;
    var numText = doc.createTextNode('[' + n + ']');

    if (node.tagName == "A") {
	if (TurnOn) {
	    node.insertBefore(numText, node.firstChild);
	} else {
	    node.removeChild(node.firstChild);
	}
    } else {
	if (TurnOn) {
	    node.parentNode.insertBefore(numText, node);
	} else {
	    node.parentNode.removeChild(node.previousSibling);
	}
    }
    } catch(e) {window.alert(e);}
}


// Handle frames if they're present
function getPageLinkNodes()
{
    var frames = window._content.frames;

    // The main content may have link nodes as well as it's frames.
    var nodes = getLinkNodes(_content.content.document);
    var tmp;
    for (var i=0; i<frames.length; i++) {
	tmp = getLinkNodes(frames[i].document);
	// is javascript this crappy?
	for (var j=0; j<tmp.length; j++)
	    nodes.push(tmp[j]);
    }
    return nodes;
}

// For a single document, grab all the nodes
function getLinkNodes(doc)
{
    var a_nodes = doc.getElementsByTagName('a');
    var i_nodes = doc.getElementsByTagName('input');
    var s_nodes = doc.getElementsByTagName('select');
    var t_nodes = doc.getElementsByTagName('textarea');

    var links = [];

    for (var i=0; i<t_nodes.length; i++) {
	links.push(t_nodes[i]);
    }
    for (var i=0; i<s_nodes.length; i++) {
	links.push(s_nodes[i]);
    }
    for (var i=0; i<i_nodes.length; i++) {
	if (i_nodes[i].type == "hidden") continue;
	links.push(i_nodes[i]);
    }
    for (var i=0; i<a_nodes.length; i++) {
	if (!a_nodes[i].hasAttribute('href')) continue;
	links.push(a_nodes[i]);
    }

    return links;
}

function setNumberedLinksState(linksOn) 
{
    try {
	var doc = _content.content.document;
	if (!doc) return;

	// Keep track of the numbered state
	if (linksOn == doc.__numberedlinks_linkState)
	    return;
	doc.__numberedlinks_linkState = linksOn;

    // accumulate our nodes
    var nodes = getPageLinkNodes();

    // Finally, give them numbers
    for (var i=0; i<nodes.length; i++) {
	toggleNum(nodes[i],i+1, linksOn);
    }

    } catch (e) {alert("toggleNumberedLinks: " + e);}
}

function toggleNumberedLinks() 
{
    var doc = _content.content.document;
    setNumberedLinksState (!doc.__numberedlinks_linkState);
}
